<?php

namespace Drupal\simple_favs\Plugin\views\field;

use Drupal\Core\Render\Markup;
use Drupal\Core\Url;
use Drupal\views\Attribute\ViewsField;
use Drupal\views\ResultRow;
use Drupal\views\Plugin\views\field\FieldPluginBase;

/**
 * A handler to provide custom displays for subscription dates.
 *
 * @ingroup views_field_handlers
 */
#[ViewsField("simple_favs_views_heart")]
class SimpleFavsViewsHeart extends FieldPluginBase {

  /**
   * {@inheritdoc}
   */
  public function render(ResultRow $values) {
    $nid = $values->_entity->id();
    if (!$nid) {
      return [];
    }
    $uid = \Drupal::currentUser()->id();
    $config = \Drupal::config('simple_favs.settings');
    $use_db = $config->get('use_database_storage');
    $fontawesome = $config->get('fontawesome_enabled') && \Drupal::moduleHandler()->moduleExists('fontawesome');

    $active = FALSE;

    if ($uid && $use_db && $nid) {
      $result = \Drupal::database()
        ->select('simple_favs_user', 's')
        ->fields('s', ['favs'])
        ->condition('uid', $uid)
        ->execute()
        ->fetchField();

      if ($result) {
        $favs = unserialize($result);
        $active = in_array($nid, $favs);
      }
    }

    $front_path = Url::fromRoute('<front>', [], ['absolute' => FALSE])->toString();
    $front_raw = Url::fromRoute('<front>', [], ['absolute' => TRUE])->toString();
    $front_scheme_and_domain = str_replace($front_path, '', $front_raw);

    $icon_type = $fontawesome ? ' font-awesome' : ' character-mode';
    $icon_markup = $fontawesome
      ? ($active ? '<i class="fas fa-heart"></i>' : '<i class="far fa-heart"></i>')
      : ($active ? '♥' : '♡');

    return [
      '#markup' => Markup::create(
        '<div class="simple-favs-views views-heart' . ($active ? ' active' : '') . $icon_type . '" data-id="' . $nid . '" role="button" tabindex="0" aria-label="Toggle Favourite" title="Toggle Favourite"><span class="heart-icon">' . $icon_markup . '</span></div>'
      ),
      '#attached' => [
        'library' => ['simple_favs/views_heart'],
        'drupalSettings' => [
          'simple_favs' => [
            'views_heart' => [
              'use_database_storage' => $use_db,
              'cookie_expiration' => $config->get('cookie_expiration'),
              'current_uid' => $uid,
              'fontawesome_enabled' => $fontawesome,
              'front_path' => $front_path,
              'front_raw' => $front_raw,
              'front_scheme_and_domain' => $front_scheme_and_domain,
            ],
          ],
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    $tags = parent::getCacheTags();
    $uid = \Drupal::currentUser()->id();
    if ($uid) {
      $tags[] = 'simple_favs:user:' . $uid;
    }
    return $tags;
  }

}

